  if (!module_african (outer)) return false;
  if (!module_afrtran (outer)) return false;
  if (!module_atarist (outer)) return false;
  if (!module_bangbang (outer)) return false;
  if (!module_cdcnos (outer)) return false;
  if (!module_ebcdic (outer)) return false;
  if (!module_ibmpc (outer)) return false;
  if (!module_iconqnx (outer)) return false;
  if (!module_latin1_ascii (outer)) return false;
  if (!module_latin1_iso5426 (outer)) return false;
  if (!module_latin1_ansel (outer)) return false;
  if (!module_java (outer)) return false;
  if (!module_mule (outer)) return false;
  if (!module_strips (outer)) return false;
  if (!module_testdump (outer)) return false;
  if (!module_ucs (outer)) return false;
  if (!module_utf16 (outer)) return false;
  if (!module_utf7 (outer)) return false;
  if (!module_utf8 (outer)) return false;
  if (!module_varia (outer)) return false;
  if (!module_vietnamese (outer)) return false;
  if (!module_flat (outer)) return false;
  if (!module_html (outer)) return false;
  if (!module_latin1_latex (outer)) return false;
  if (!module_latin1_bibtex (outer)) return false;
  if (!module_latin1_texte (outer)) return false;
  if (!module_rfc1345 (outer)) return false;
  if (!module_texinfo (outer)) return false;
  if (!module_base64 (outer)) return false;
  if (!module_dump (outer)) return false;
  if (!module_endline (outer)) return false;
  if (!module_permutations (outer)) return false;
  if (!module_quoted_printable (outer)) return false;
  if (!module_ascii_latin1 (outer)) return false;
  if (!module_iso5426_latin1 (outer)) return false;
  if (!module_ansel_latin1 (outer)) return false;
  if (!module_latex_latin1 (outer)) return false;
  if (!module_bibtex_latin1 (outer)) return false;
  if (!module_texte_latin1 (outer)) return false;
