  delmodule_african (outer);
  delmodule_afrtran (outer);
  delmodule_atarist (outer);
  delmodule_bangbang (outer);
  delmodule_cdcnos (outer);
  delmodule_ebcdic (outer);
  delmodule_ibmpc (outer);
  delmodule_iconqnx (outer);
  delmodule_latin1_ascii (outer);
  delmodule_latin1_iso5426 (outer);
  delmodule_latin1_ansel (outer);
  delmodule_java (outer);
  delmodule_mule (outer);
  delmodule_strips (outer);
  delmodule_testdump (outer);
  delmodule_ucs (outer);
  delmodule_utf16 (outer);
  delmodule_utf7 (outer);
  delmodule_utf8 (outer);
  delmodule_varia (outer);
  delmodule_vietnamese (outer);
  delmodule_flat (outer);
  delmodule_html (outer);
  delmodule_latin1_latex (outer);
  delmodule_latin1_bibtex (outer);
  delmodule_latin1_texte (outer);
  delmodule_rfc1345 (outer);
  delmodule_texinfo (outer);
  delmodule_base64 (outer);
  delmodule_dump (outer);
  delmodule_endline (outer);
  delmodule_permutations (outer);
  delmodule_quoted_printable (outer);
  delmodule_ascii_latin1 (outer);
  delmodule_iso5426_latin1 (outer);
  delmodule_ansel_latin1 (outer);
  delmodule_latex_latin1 (outer);
  delmodule_bibtex_latin1 (outer);
  delmodule_texte_latin1 (outer);
