/* DO NOT MODIFY THIS FILE!  It was generated by `recode/tables.py'.  */

/* Conversion of files between different charsets and surfaces.
   Copyright © 1999 Free Software Foundation, Inc.
   Contributed by François Pinard <pinard@iro.umontreal.ca>, 1993, 1997.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 3 of the
   License, or (at your option) any later version.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the Recode Library; see the file `COPYING.LIB'.
   If not, see <https://www.gnu.org/licenses/>.
   Suite 330, Boston, MA 02111-1307, USA.  */

/* This is derived from Bruno Haible's `libiconv' package.  */
static const char *iconv_name_list[4701] =
  {
    "437", NULL,
    "437-translit", NULL,
    "500", NULL,
    "500-translit", NULL,
    "500V1", NULL,
    "500V1-translit", NULL,
    "850", NULL,
    "850-translit", NULL,
    "851", NULL,
    "851-translit", NULL,
    "852", NULL,
    "852-translit", NULL,
    "855", NULL,
    "855-translit", NULL,
    "856", NULL,
    "856-translit", NULL,
    "857", NULL,
    "857-translit", NULL,
    "858", NULL,
    "858-translit", NULL,
    "860", NULL,
    "860-translit", NULL,
    "861", NULL,
    "861-translit", NULL,
    "862", NULL,
    "862-translit", NULL,
    "863", NULL,
    "863-translit", NULL,
    "864", NULL,
    "864-translit", NULL,
    "865", NULL,
    "865-translit", NULL,
    "866", NULL,
    "866-translit", NULL,
    "866NAV", NULL,
    "866NAV-translit", NULL,
    "869", NULL,
    "869-translit", NULL,
    "874", NULL,
    "874-translit", NULL,
    "904", NULL,
    "904-translit", NULL,
    "1026", NULL,
    "1026-translit", NULL,
    "1046", NULL,
    "1046-translit", NULL,
    "1047", NULL,
    "1047-translit", NULL,
    "8859_1", NULL,
    "8859_1-translit", NULL,
    "8859_2", NULL,
    "8859_2-translit", NULL,
    "8859_3", NULL,
    "8859_3-translit", NULL,
    "8859_4", NULL,
    "8859_4-translit", NULL,
    "8859_5", NULL,
    "8859_5-translit", NULL,
    "8859_6", NULL,
    "8859_6-translit", NULL,
    "8859_7", NULL,
    "8859_7-translit", NULL,
    "8859_8", NULL,
    "8859_8-translit", NULL,
    "8859_9", NULL,
    "8859_9-translit", NULL,
    "10646-1:1993", NULL,
    "10646-1:1993-translit", NULL,
    "UCS4", NULL,
    "UCS4-translit", NULL,
    "ANSI_X3.4-1968", NULL,
    "ANSI_X3.4-1968-translit", NULL,
    "ANSI_X3.4-1986", NULL,
    "ANSI_X3.4-1986-translit", NULL,
    "ANSI_X3.4", NULL,
    "ANSI_X3.4-translit", NULL,
    "ANSI_X3.110-1983", NULL,
    "ANSI_X3.110-1983-translit", NULL,
    "ANSI_X3.110", NULL,
    "ANSI_X3.110-translit", NULL,
    "ARABIC", NULL,
    "ARABIC-translit", NULL,
    "ARABIC7", NULL,
    "ARABIC7-translit", NULL,
    "ARMSCII-8", NULL,
    "ARMSCII-8-translit", NULL,
    "ARMSCII8", NULL,
    "ARMSCII8-translit", NULL,
    "ASCII", NULL,
    "ASCII-translit", NULL,
    "ASMO-708", NULL,
    "ASMO-708-translit", NULL,
    "ASMO_449", NULL,
    "ASMO_449-translit", NULL,
    "BALTIC", NULL,
    "BALTIC-translit", NULL,
    "BIG-5", NULL,
    "BIG-5-translit", NULL,
    "BIG-FIVE", NULL,
    "BIG-FIVE-translit", NULL,
    "BIG5-HKSCS", NULL,
    "BIG5-HKSCS-translit", NULL,
    "BIG5", NULL,
    "BIG5-translit", NULL,
    "BIG5HKSCS", NULL,
    "BIG5HKSCS-translit", NULL,
    "BIGFIVE", NULL,
    "BIGFIVE-translit", NULL,
    "BRF", NULL,
    "BRF-translit", NULL,
    "BS_4730", NULL,
    "BS_4730-translit", NULL,
    "CA", NULL,
    "CA-translit", NULL,
    "CN-BIG5", NULL,
    "CN-BIG5-translit", NULL,
    "CN-GB", NULL,
    "CN-GB-translit", NULL,
    "CN", NULL,
    "CN-translit", NULL,
    "CP-AR", NULL,
    "CP-AR-translit", NULL,
    "CP-GR", NULL,
    "CP-GR-translit", NULL,
    "CP-HU", NULL,
    "CP-HU-translit", NULL,
    "CP037", NULL,
    "CP037-translit", NULL,
    "CP038", NULL,
    "CP038-translit", NULL,
    "CP273", NULL,
    "CP273-translit", NULL,
    "CP274", NULL,
    "CP274-translit", NULL,
    "CP275", NULL,
    "CP275-translit", NULL,
    "CP278", NULL,
    "CP278-translit", NULL,
    "CP280", NULL,
    "CP280-translit", NULL,
    "CP281", NULL,
    "CP281-translit", NULL,
    "CP282", NULL,
    "CP282-translit", NULL,
    "CP284", NULL,
    "CP284-translit", NULL,
    "CP285", NULL,
    "CP285-translit", NULL,
    "CP290", NULL,
    "CP290-translit", NULL,
    "CP297", NULL,
    "CP297-translit", NULL,
    "CP367", NULL,
    "CP367-translit", NULL,
    "CP420", NULL,
    "CP420-translit", NULL,
    "CP423", NULL,
    "CP423-translit", NULL,
    "CP424", NULL,
    "CP424-translit", NULL,
    "CP437", NULL,
    "CP437-translit", NULL,
    "CP500", NULL,
    "CP500-translit", NULL,
    "CP737", NULL,
    "CP737-translit", NULL,
    "CP770", NULL,
    "CP770-translit", NULL,
    "CP771", NULL,
    "CP771-translit", NULL,
    "CP772", NULL,
    "CP772-translit", NULL,
    "CP773", NULL,
    "CP773-translit", NULL,
    "CP774", NULL,
    "CP774-translit", NULL,
    "CP775", NULL,
    "CP775-translit", NULL,
    "CP803", NULL,
    "CP803-translit", NULL,
    "CP813", NULL,
    "CP813-translit", NULL,
    "CP819", NULL,
    "CP819-translit", NULL,
    "CP850", NULL,
    "CP850-translit", NULL,
    "CP851", NULL,
    "CP851-translit", NULL,
    "CP852", NULL,
    "CP852-translit", NULL,
    "CP855", NULL,
    "CP855-translit", NULL,
    "CP856", NULL,
    "CP856-translit", NULL,
    "CP857", NULL,
    "CP857-translit", NULL,
    "CP858", NULL,
    "CP858-translit", NULL,
    "CP860", NULL,
    "CP860-translit", NULL,
    "CP861", NULL,
    "CP861-translit", NULL,
    "CP862", NULL,
    "CP862-translit", NULL,
    "CP863", NULL,
    "CP863-translit", NULL,
    "CP864", NULL,
    "CP864-translit", NULL,
    "CP865", NULL,
    "CP865-translit", NULL,
    "CP866", NULL,
    "CP866-translit", NULL,
    "CP866NAV", NULL,
    "CP866NAV-translit", NULL,
    "CP868", NULL,
    "CP868-translit", NULL,
    "CP869", NULL,
    "CP869-translit", NULL,
    "CP870", NULL,
    "CP870-translit", NULL,
    "CP871", NULL,
    "CP871-translit", NULL,
    "CP874", NULL,
    "CP874-translit", NULL,
    "CP875", NULL,
    "CP875-translit", NULL,
    "CP880", NULL,
    "CP880-translit", NULL,
    "CP891", NULL,
    "CP891-translit", NULL,
    "CP901", NULL,
    "CP901-translit", NULL,
    "CP902", NULL,
    "CP902-translit", NULL,
    "CP903", NULL,
    "CP903-translit", NULL,
    "CP904", NULL,
    "CP904-translit", NULL,
    "CP905", NULL,
    "CP905-translit", NULL,
    "CP912", NULL,
    "CP912-translit", NULL,
    "CP915", NULL,
    "CP915-translit", NULL,
    "CP916", NULL,
    "CP916-translit", NULL,
    "CP918", NULL,
    "CP918-translit", NULL,
    "CP920", NULL,
    "CP920-translit", NULL,
    "CP921", NULL,
    "CP921-translit", NULL,
    "CP922", NULL,
    "CP922-translit", NULL,
    "CP930", NULL,
    "CP930-translit", NULL,
    "CP932", NULL,
    "CP932-translit", NULL,
    "CP933", NULL,
    "CP933-translit", NULL,
    "CP935", NULL,
    "CP935-translit", NULL,
    "CP936", NULL,
    "CP936-translit", NULL,
    "CP937", NULL,
    "CP937-translit", NULL,
    "CP939", NULL,
    "CP939-translit", NULL,
    "CP949", NULL,
    "CP949-translit", NULL,
    "CP950", NULL,
    "CP950-translit", NULL,
    "CP1004", NULL,
    "CP1004-translit", NULL,
    "CP1008", NULL,
    "CP1008-translit", NULL,
    "CP1025", NULL,
    "CP1025-translit", NULL,
    "CP1026", NULL,
    "CP1026-translit", NULL,
    "CP1046", NULL,
    "CP1046-translit", NULL,
    "CP1047", NULL,
    "CP1047-translit", NULL,
    "CP1070", NULL,
    "CP1070-translit", NULL,
    "CP1079", NULL,
    "CP1079-translit", NULL,
    "CP1081", NULL,
    "CP1081-translit", NULL,
    "CP1084", NULL,
    "CP1084-translit", NULL,
    "CP1089", NULL,
    "CP1089-translit", NULL,
    "CP1097", NULL,
    "CP1097-translit", NULL,
    "CP1112", NULL,
    "CP1112-translit", NULL,
    "CP1122", NULL,
    "CP1122-translit", NULL,
    "CP1123", NULL,
    "CP1123-translit", NULL,
    "CP1124", NULL,
    "CP1124-translit", NULL,
    "CP1125", NULL,
    "CP1125-translit", NULL,
    "CP1129", NULL,
    "CP1129-translit", NULL,
    "CP1130", NULL,
    "CP1130-translit", NULL,
    "CP1132", NULL,
    "CP1132-translit", NULL,
    "CP1133", NULL,
    "CP1133-translit", NULL,
    "CP1137", NULL,
    "CP1137-translit", NULL,
    "CP1140", NULL,
    "CP1140-translit", NULL,
    "CP1141", NULL,
    "CP1141-translit", NULL,
    "CP1142", NULL,
    "CP1142-translit", NULL,
    "CP1143", NULL,
    "CP1143-translit", NULL,
    "CP1144", NULL,
    "CP1144-translit", NULL,
    "CP1145", NULL,
    "CP1145-translit", NULL,
    "CP1146", NULL,
    "CP1146-translit", NULL,
    "CP1147", NULL,
    "CP1147-translit", NULL,
    "CP1148", NULL,
    "CP1148-translit", NULL,
    "CP1149", NULL,
    "CP1149-translit", NULL,
    "CP1153", NULL,
    "CP1153-translit", NULL,
    "CP1154", NULL,
    "CP1154-translit", NULL,
    "CP1155", NULL,
    "CP1155-translit", NULL,
    "CP1156", NULL,
    "CP1156-translit", NULL,
    "CP1157", NULL,
    "CP1157-translit", NULL,
    "CP1158", NULL,
    "CP1158-translit", NULL,
    "CP1160", NULL,
    "CP1160-translit", NULL,
    "CP1161", NULL,
    "CP1161-translit", NULL,
    "CP1162", NULL,
    "CP1162-translit", NULL,
    "CP1163", NULL,
    "CP1163-translit", NULL,
    "CP1164", NULL,
    "CP1164-translit", NULL,
    "CP1166", NULL,
    "CP1166-translit", NULL,
    "CP1167", NULL,
    "CP1167-translit", NULL,
    "CP1250", NULL,
    "CP1250-translit", NULL,
    "CP1251", NULL,
    "CP1251-translit", NULL,
    "CP1252", NULL,
    "CP1252-translit", NULL,
    "CP1253", NULL,
    "CP1253-translit", NULL,
    "CP1254", NULL,
    "CP1254-translit", NULL,
    "CP1255", NULL,
    "CP1255-translit", NULL,
    "CP1256", NULL,
    "CP1256-translit", NULL,
    "CP1257", NULL,
    "CP1257-translit", NULL,
    "CP1258", NULL,
    "CP1258-translit", NULL,
    "CP1282", NULL,
    "CP1282-translit", NULL,
    "CP1361", NULL,
    "CP1361-translit", NULL,
    "CP1364", NULL,
    "CP1364-translit", NULL,
    "CP1371", NULL,
    "CP1371-translit", NULL,
    "CP1388", NULL,
    "CP1388-translit", NULL,
    "CP1390", NULL,
    "CP1390-translit", NULL,
    "CP1399", NULL,
    "CP1399-translit", NULL,
    "CP4517", NULL,
    "CP4517-translit", NULL,
    "CP4899", NULL,
    "CP4899-translit", NULL,
    "CP4909", NULL,
    "CP4909-translit", NULL,
    "CP4971", NULL,
    "CP4971-translit", NULL,
    "CP5347", NULL,
    "CP5347-translit", NULL,
    "CP9030", NULL,
    "CP9030-translit", NULL,
    "CP9066", NULL,
    "CP9066-translit", NULL,
    "CP9448", NULL,
    "CP9448-translit", NULL,
    "CP10007", NULL,
    "CP10007-translit", NULL,
    "CP12712", NULL,
    "CP12712-translit", NULL,
    "CP16804", NULL,
    "CP16804-translit", NULL,
    "CPIBM861", NULL,
    "CPIBM861-translit", NULL,
    "CSA7-1", NULL,
    "CSA7-1-translit", NULL,
    "CSA7-2", NULL,
    "CSA7-2-translit", NULL,
    "CSASCII", NULL,
    "CSASCII-translit", NULL,
    "CSA_T500-1983", NULL,
    "CSA_T500-1983-translit", NULL,
    "CSA_T500", NULL,
    "CSA_T500-translit", NULL,
    "CSA_Z243.4-1985-1", NULL,
    "CSA_Z243.4-1985-1-translit", NULL,
    "CSA_Z243.4-1985-2", NULL,
    "CSA_Z243.4-1985-2-translit", NULL,
    "CSA_Z243.419851", NULL,
    "CSA_Z243.419851-translit", NULL,
    "CSA_Z243.419852", NULL,
    "CSA_Z243.419852-translit", NULL,
    "CSDECMCS", NULL,
    "CSDECMCS-translit", NULL,
    "CSEBCDICATDE", NULL,
    "CSEBCDICATDE-translit", NULL,
    "CSEBCDICATDEA", NULL,
    "CSEBCDICATDEA-translit", NULL,
    "CSEBCDICCAFR", NULL,
    "CSEBCDICCAFR-translit", NULL,
    "CSEBCDICDKNO", NULL,
    "CSEBCDICDKNO-translit", NULL,
    "CSEBCDICDKNOA", NULL,
    "CSEBCDICDKNOA-translit", NULL,
    "CSEBCDICES", NULL,
    "CSEBCDICES-translit", NULL,
    "CSEBCDICESA", NULL,
    "CSEBCDICESA-translit", NULL,
    "CSEBCDICESS", NULL,
    "CSEBCDICESS-translit", NULL,
    "CSEBCDICFISE", NULL,
    "CSEBCDICFISE-translit", NULL,
    "CSEBCDICFISEA", NULL,
    "CSEBCDICFISEA-translit", NULL,
    "CSEBCDICFR", NULL,
    "CSEBCDICFR-translit", NULL,
    "CSEBCDICIT", NULL,
    "CSEBCDICIT-translit", NULL,
    "CSEBCDICPT", NULL,
    "CSEBCDICPT-translit", NULL,
    "CSEBCDICUK", NULL,
    "CSEBCDICUK-translit", NULL,
    "CSEBCDICUS", NULL,
    "CSEBCDICUS-translit", NULL,
    "CSEUCKR", NULL,
    "CSEUCKR-translit", NULL,
    "CSEUCPKDFMTJAPANESE", NULL,
    "CSEUCPKDFMTJAPANESE-translit", NULL,
    "CSGB2312", NULL,
    "CSGB2312-translit", NULL,
    "CSHPROMAN8", NULL,
    "CSHPROMAN8-translit", NULL,
    "CSIBM037", NULL,
    "CSIBM037-translit", NULL,
    "CSIBM038", NULL,
    "CSIBM038-translit", NULL,
    "CSIBM273", NULL,
    "CSIBM273-translit", NULL,
    "CSIBM274", NULL,
    "CSIBM274-translit", NULL,
    "CSIBM275", NULL,
    "CSIBM275-translit", NULL,
    "CSIBM277", NULL,
    "CSIBM277-translit", NULL,
    "CSIBM278", NULL,
    "CSIBM278-translit", NULL,
    "CSIBM280", NULL,
    "CSIBM280-translit", NULL,
    "CSIBM281", NULL,
    "CSIBM281-translit", NULL,
    "CSIBM284", NULL,
    "CSIBM284-translit", NULL,
    "CSIBM285", NULL,
    "CSIBM285-translit", NULL,
    "CSIBM290", NULL,
    "CSIBM290-translit", NULL,
    "CSIBM297", NULL,
    "CSIBM297-translit", NULL,
    "CSIBM420", NULL,
    "CSIBM420-translit", NULL,
    "CSIBM423", NULL,
    "CSIBM423-translit", NULL,
    "CSIBM424", NULL,
    "CSIBM424-translit", NULL,
    "CSIBM500", NULL,
    "CSIBM500-translit", NULL,
    "CSIBM803", NULL,
    "CSIBM803-translit", NULL,
    "CSIBM851", NULL,
    "CSIBM851-translit", NULL,
    "CSIBM855", NULL,
    "CSIBM855-translit", NULL,
    "CSIBM856", NULL,
    "CSIBM856-translit", NULL,
    "CSIBM857", NULL,
    "CSIBM857-translit", NULL,
    "CSIBM860", NULL,
    "CSIBM860-translit", NULL,
    "CSIBM863", NULL,
    "CSIBM863-translit", NULL,
    "CSIBM864", NULL,
    "CSIBM864-translit", NULL,
    "CSIBM865", NULL,
    "CSIBM865-translit", NULL,
    "CSIBM866", NULL,
    "CSIBM866-translit", NULL,
    "CSIBM868", NULL,
    "CSIBM868-translit", NULL,
    "CSIBM869", NULL,
    "CSIBM869-translit", NULL,
    "CSIBM870", NULL,
    "CSIBM870-translit", NULL,
    "CSIBM871", NULL,
    "CSIBM871-translit", NULL,
    "CSIBM880", NULL,
    "CSIBM880-translit", NULL,
    "CSIBM891", NULL,
    "CSIBM891-translit", NULL,
    "CSIBM901", NULL,
    "CSIBM901-translit", NULL,
    "CSIBM902", NULL,
    "CSIBM902-translit", NULL,
    "CSIBM903", NULL,
    "CSIBM903-translit", NULL,
    "CSIBM904", NULL,
    "CSIBM904-translit", NULL,
    "CSIBM905", NULL,
    "CSIBM905-translit", NULL,
    "CSIBM918", NULL,
    "CSIBM918-translit", NULL,
    "CSIBM921", NULL,
    "CSIBM921-translit", NULL,
    "CSIBM922", NULL,
    "CSIBM922-translit", NULL,
    "CSIBM930", NULL,
    "CSIBM930-translit", NULL,
    "CSIBM932", NULL,
    "CSIBM932-translit", NULL,
    "CSIBM933", NULL,
    "CSIBM933-translit", NULL,
    "CSIBM935", NULL,
    "CSIBM935-translit", NULL,
    "CSIBM937", NULL,
    "CSIBM937-translit", NULL,
    "CSIBM939", NULL,
    "CSIBM939-translit", NULL,
    "CSIBM943", NULL,
    "CSIBM943-translit", NULL,
    "CSIBM1008", NULL,
    "CSIBM1008-translit", NULL,
    "CSIBM1025", NULL,
    "CSIBM1025-translit", NULL,
    "CSIBM1026", NULL,
    "CSIBM1026-translit", NULL,
    "CSIBM1097", NULL,
    "CSIBM1097-translit", NULL,
    "CSIBM1112", NULL,
    "CSIBM1112-translit", NULL,
    "CSIBM1122", NULL,
    "CSIBM1122-translit", NULL,
    "CSIBM1123", NULL,
    "CSIBM1123-translit", NULL,
    "CSIBM1124", NULL,
    "CSIBM1124-translit", NULL,
    "CSIBM1129", NULL,
    "CSIBM1129-translit", NULL,
    "CSIBM1130", NULL,
    "CSIBM1130-translit", NULL,
    "CSIBM1132", NULL,
    "CSIBM1132-translit", NULL,
    "CSIBM1133", NULL,
    "CSIBM1133-translit", NULL,
    "CSIBM1137", NULL,
    "CSIBM1137-translit", NULL,
    "CSIBM1140", NULL,
    "CSIBM1140-translit", NULL,
    "CSIBM1141", NULL,
    "CSIBM1141-translit", NULL,
    "CSIBM1142", NULL,
    "CSIBM1142-translit", NULL,
    "CSIBM1143", NULL,
    "CSIBM1143-translit", NULL,
    "CSIBM1144", NULL,
    "CSIBM1144-translit", NULL,
    "CSIBM1145", NULL,
    "CSIBM1145-translit", NULL,
    "CSIBM1146", NULL,
    "CSIBM1146-translit", NULL,
    "CSIBM1147", NULL,
    "CSIBM1147-translit", NULL,
    "CSIBM1148", NULL,
    "CSIBM1148-translit", NULL,
    "CSIBM1149", NULL,
    "CSIBM1149-translit", NULL,
    "CSIBM1153", NULL,
    "CSIBM1153-translit", NULL,
    "CSIBM1154", NULL,
    "CSIBM1154-translit", NULL,
    "CSIBM1155", NULL,
    "CSIBM1155-translit", NULL,
    "CSIBM1156", NULL,
    "CSIBM1156-translit", NULL,
    "CSIBM1157", NULL,
    "CSIBM1157-translit", NULL,
    "CSIBM1158", NULL,
    "CSIBM1158-translit", NULL,
    "CSIBM1160", NULL,
    "CSIBM1160-translit", NULL,
    "CSIBM1161", NULL,
    "CSIBM1161-translit", NULL,
    "CSIBM1163", NULL,
    "CSIBM1163-translit", NULL,
    "CSIBM1164", NULL,
    "CSIBM1164-translit", NULL,
    "CSIBM1166", NULL,
    "CSIBM1166-translit", NULL,
    "CSIBM1167", NULL,
    "CSIBM1167-translit", NULL,
    "CSIBM1364", NULL,
    "CSIBM1364-translit", NULL,
    "CSIBM1371", NULL,
    "CSIBM1371-translit", NULL,
    "CSIBM1388", NULL,
    "CSIBM1388-translit", NULL,
    "CSIBM1390", NULL,
    "CSIBM1390-translit", NULL,
    "CSIBM1399", NULL,
    "CSIBM1399-translit", NULL,
    "CSIBM4517", NULL,
    "CSIBM4517-translit", NULL,
    "CSIBM4899", NULL,
    "CSIBM4899-translit", NULL,
    "CSIBM4909", NULL,
    "CSIBM4909-translit", NULL,
    "CSIBM4971", NULL,
    "CSIBM4971-translit", NULL,
    "CSIBM5347", NULL,
    "CSIBM5347-translit", NULL,
    "CSIBM9030", NULL,
    "CSIBM9030-translit", NULL,
    "CSIBM9066", NULL,
    "CSIBM9066-translit", NULL,
    "CSIBM9448", NULL,
    "CSIBM9448-translit", NULL,
    "CSIBM12712", NULL,
    "CSIBM12712-translit", NULL,
    "CSIBM16804", NULL,
    "CSIBM16804-translit", NULL,
    "CSIBM11621162", NULL,
    "CSIBM11621162-translit", NULL,
    "CSISO4UNITEDKINGDOM", NULL,
    "CSISO4UNITEDKINGDOM-translit", NULL,
    "CSISO10SWEDISH", NULL,
    "CSISO10SWEDISH-translit", NULL,
    "CSISO11SWEDISHFORNAMES", NULL,
    "CSISO11SWEDISHFORNAMES-translit", NULL,
    "CSISO14JISC6220RO", NULL,
    "CSISO14JISC6220RO-translit", NULL,
    "CSISO15ITALIAN", NULL,
    "CSISO15ITALIAN-translit", NULL,
    "CSISO16PORTUGESE", NULL,
    "CSISO16PORTUGESE-translit", NULL,
    "CSISO17SPANISH", NULL,
    "CSISO17SPANISH-translit", NULL,
    "CSISO18GREEK7OLD", NULL,
    "CSISO18GREEK7OLD-translit", NULL,
    "CSISO19LATINGREEK", NULL,
    "CSISO19LATINGREEK-translit", NULL,
    "CSISO21GERMAN", NULL,
    "CSISO21GERMAN-translit", NULL,
    "CSISO25FRENCH", NULL,
    "CSISO25FRENCH-translit", NULL,
    "CSISO27LATINGREEK1", NULL,
    "CSISO27LATINGREEK1-translit", NULL,
    "CSISO49INIS", NULL,
    "CSISO49INIS-translit", NULL,
    "CSISO50INIS8", NULL,
    "CSISO50INIS8-translit", NULL,
    "CSISO51INISCYRILLIC", NULL,
    "CSISO51INISCYRILLIC-translit", NULL,
    "CSISO58GB1988", NULL,
    "CSISO58GB1988-translit", NULL,
    "CSISO60DANISHNORWEGIAN", NULL,
    "CSISO60DANISHNORWEGIAN-translit", NULL,
    "CSISO60NORWEGIAN1", NULL,
    "CSISO60NORWEGIAN1-translit", NULL,
    "CSISO61NORWEGIAN2", NULL,
    "CSISO61NORWEGIAN2-translit", NULL,
    "CSISO69FRENCH", NULL,
    "CSISO69FRENCH-translit", NULL,
    "CSISO84PORTUGUESE2", NULL,
    "CSISO84PORTUGUESE2-translit", NULL,
    "CSISO85SPANISH2", NULL,
    "CSISO85SPANISH2-translit", NULL,
    "CSISO86HUNGARIAN", NULL,
    "CSISO86HUNGARIAN-translit", NULL,
    "CSISO88GREEK7", NULL,
    "CSISO88GREEK7-translit", NULL,
    "CSISO89ASMO449", NULL,
    "CSISO89ASMO449-translit", NULL,
    "CSISO90", NULL,
    "CSISO90-translit", NULL,
    "CSISO92JISC62991984B", NULL,
    "CSISO92JISC62991984B-translit", NULL,
    "CSISO99NAPLPS", NULL,
    "CSISO99NAPLPS-translit", NULL,
    "CSISO103T618BIT", NULL,
    "CSISO103T618BIT-translit", NULL,
    "CSISO111ECMACYRILLIC", NULL,
    "CSISO111ECMACYRILLIC-translit", NULL,
    "CSISO121CANADIAN1", NULL,
    "CSISO121CANADIAN1-translit", NULL,
    "CSISO122CANADIAN2", NULL,
    "CSISO122CANADIAN2-translit", NULL,
    "CSISO139CSN369103", NULL,
    "CSISO139CSN369103-translit", NULL,
    "CSISO141JUSIB1002", NULL,
    "CSISO141JUSIB1002-translit", NULL,
    "CSISO143IECP271", NULL,
    "CSISO143IECP271-translit", NULL,
    "CSISO150", NULL,
    "CSISO150-translit", NULL,
    "CSISO150GREEKCCITT", NULL,
    "CSISO150GREEKCCITT-translit", NULL,
    "CSISO151CUBA", NULL,
    "CSISO151CUBA-translit", NULL,
    "CSISO153GOST1976874", NULL,
    "CSISO153GOST1976874-translit", NULL,
    "CSISO646DANISH", NULL,
    "CSISO646DANISH-translit", NULL,
    "CSISO2022CN", NULL,
    "CSISO2022CN-translit", NULL,
    "CSISO2022JP", NULL,
    "CSISO2022JP-translit", NULL,
    "CSISO2022JP2", NULL,
    "CSISO2022JP2-translit", NULL,
    "CSISO2022KR", NULL,
    "CSISO2022KR-translit", NULL,
    "CSISO2033", NULL,
    "CSISO2033-translit", NULL,
    "CSISO5427CYRILLIC", NULL,
    "CSISO5427CYRILLIC-translit", NULL,
    "CSISO5427CYRILLIC1981", NULL,
    "CSISO5427CYRILLIC1981-translit", NULL,
    "CSISO5428GREEK", NULL,
    "CSISO5428GREEK-translit", NULL,
    "CSISO10367BOX", NULL,
    "CSISO10367BOX-translit", NULL,
    "CSISOLATIN1", NULL,
    "CSISOLATIN1-translit", NULL,
    "CSISOLATIN2", NULL,
    "CSISOLATIN2-translit", NULL,
    "CSISOLATIN3", NULL,
    "CSISOLATIN3-translit", NULL,
    "CSISOLATIN4", NULL,
    "CSISOLATIN4-translit", NULL,
    "CSISOLATIN5", NULL,
    "CSISOLATIN5-translit", NULL,
    "CSISOLATIN6", NULL,
    "CSISOLATIN6-translit", NULL,
    "CSISOLATINARABIC", NULL,
    "CSISOLATINARABIC-translit", NULL,
    "CSISOLATINCYRILLIC", NULL,
    "CSISOLATINCYRILLIC-translit", NULL,
    "CSISOLATINGREEK", NULL,
    "CSISOLATINGREEK-translit", NULL,
    "CSISOLATINHEBREW", NULL,
    "CSISOLATINHEBREW-translit", NULL,
    "CSKOI8R", NULL,
    "CSKOI8R-translit", NULL,
    "CSKSC5636", NULL,
    "CSKSC5636-translit", NULL,
    "CSMACINTOSH", NULL,
    "CSMACINTOSH-translit", NULL,
    "CSNATSDANO", NULL,
    "CSNATSDANO-translit", NULL,
    "CSNATSSEFI", NULL,
    "CSNATSSEFI-translit", NULL,
    "CSN_369103", NULL,
    "CSN_369103-translit", NULL,
    "CSPC8CODEPAGE437", NULL,
    "CSPC8CODEPAGE437-translit", NULL,
    "CSPC775BALTIC", NULL,
    "CSPC775BALTIC-translit", NULL,
    "CSPC850MULTILINGUAL", NULL,
    "CSPC850MULTILINGUAL-translit", NULL,
    "CSPC858MULTILINGUAL", NULL,
    "CSPC858MULTILINGUAL-translit", NULL,
    "CSPC862LATINHEBREW", NULL,
    "CSPC862LATINHEBREW-translit", NULL,
    "CSPCP852", NULL,
    "CSPCP852-translit", NULL,
    "CSSHIFTJIS", NULL,
    "CSSHIFTJIS-translit", NULL,
    "CSUCS4", NULL,
    "CSUCS4-translit", NULL,
    "CSUNICODE", NULL,
    "CSUNICODE-translit", NULL,
    "CSWINDOWS31J", NULL,
    "CSWINDOWS31J-translit", NULL,
    "CUBA", NULL,
    "CUBA-translit", NULL,
    "CWI-2", NULL,
    "CWI-2-translit", NULL,
    "CWI", NULL,
    "CWI-translit", NULL,
    "CYRILLIC", NULL,
    "CYRILLIC-translit", NULL,
    "DE", NULL,
    "DE-translit", NULL,
    "DEC-MCS", NULL,
    "DEC-MCS-translit", NULL,
    "DEC", NULL,
    "DEC-translit", NULL,
    "DECMCS", NULL,
    "DECMCS-translit", NULL,
    "DIN_66003", NULL,
    "DIN_66003-translit", NULL,
    "DK", NULL,
    "DK-translit", NULL,
    "DS2089", NULL,
    "DS2089-translit", NULL,
    "DS_2089", NULL,
    "DS_2089-translit", NULL,
    "E13B", NULL,
    "E13B-translit", NULL,
    "EBCDIC-AT-DE-A", NULL,
    "EBCDIC-AT-DE-A-translit", NULL,
    "EBCDIC-AT-DE", NULL,
    "EBCDIC-AT-DE-translit", NULL,
    "EBCDIC-BE", NULL,
    "EBCDIC-BE-translit", NULL,
    "EBCDIC-BR", NULL,
    "EBCDIC-BR-translit", NULL,
    "EBCDIC-CA-FR", NULL,
    "EBCDIC-CA-FR-translit", NULL,
    "EBCDIC-CP-AR1", NULL,
    "EBCDIC-CP-AR1-translit", NULL,
    "EBCDIC-CP-AR2", NULL,
    "EBCDIC-CP-AR2-translit", NULL,
    "EBCDIC-CP-BE", NULL,
    "EBCDIC-CP-BE-translit", NULL,
    "EBCDIC-CP-CA", NULL,
    "EBCDIC-CP-CA-translit", NULL,
    "EBCDIC-CP-CH", NULL,
    "EBCDIC-CP-CH-translit", NULL,
    "EBCDIC-CP-DK", NULL,
    "EBCDIC-CP-DK-translit", NULL,
    "EBCDIC-CP-ES", NULL,
    "EBCDIC-CP-ES-translit", NULL,
    "EBCDIC-CP-FI", NULL,
    "EBCDIC-CP-FI-translit", NULL,
    "EBCDIC-CP-FR", NULL,
    "EBCDIC-CP-FR-translit", NULL,
    "EBCDIC-CP-GB", NULL,
    "EBCDIC-CP-GB-translit", NULL,
    "EBCDIC-CP-GR", NULL,
    "EBCDIC-CP-GR-translit", NULL,
    "EBCDIC-CP-HE", NULL,
    "EBCDIC-CP-HE-translit", NULL,
    "EBCDIC-CP-IS", NULL,
    "EBCDIC-CP-IS-translit", NULL,
    "EBCDIC-CP-IT", NULL,
    "EBCDIC-CP-IT-translit", NULL,
    "EBCDIC-CP-NL", NULL,
    "EBCDIC-CP-NL-translit", NULL,
    "EBCDIC-CP-NO", NULL,
    "EBCDIC-CP-NO-translit", NULL,
    "EBCDIC-CP-ROECE", NULL,
    "EBCDIC-CP-ROECE-translit", NULL,
    "EBCDIC-CP-SE", NULL,
    "EBCDIC-CP-SE-translit", NULL,
    "EBCDIC-CP-TR", NULL,
    "EBCDIC-CP-TR-translit", NULL,
    "EBCDIC-CP-US", NULL,
    "EBCDIC-CP-US-translit", NULL,
    "EBCDIC-CP-WT", NULL,
    "EBCDIC-CP-WT-translit", NULL,
    "EBCDIC-CP-YU", NULL,
    "EBCDIC-CP-YU-translit", NULL,
    "EBCDIC-CYRILLIC", NULL,
    "EBCDIC-CYRILLIC-translit", NULL,
    "EBCDIC-DK-NO-A", NULL,
    "EBCDIC-DK-NO-A-translit", NULL,
    "EBCDIC-DK-NO", NULL,
    "EBCDIC-DK-NO-translit", NULL,
    "EBCDIC-ES-A", NULL,
    "EBCDIC-ES-A-translit", NULL,
    "EBCDIC-ES-S", NULL,
    "EBCDIC-ES-S-translit", NULL,
    "EBCDIC-ES", NULL,
    "EBCDIC-ES-translit", NULL,
    "EBCDIC-FI-SE-A", NULL,
    "EBCDIC-FI-SE-A-translit", NULL,
    "EBCDIC-FI-SE", NULL,
    "EBCDIC-FI-SE-translit", NULL,
    "EBCDIC-FR", NULL,
    "EBCDIC-FR-translit", NULL,
    "EBCDIC-GREEK", NULL,
    "EBCDIC-GREEK-translit", NULL,
    "EBCDIC-INT", NULL,
    "EBCDIC-INT-translit", NULL,
    "EBCDIC-INT1", NULL,
    "EBCDIC-INT1-translit", NULL,
    "EBCDIC-IS-FRISS", NULL,
    "EBCDIC-IS-FRISS-translit", NULL,
    "EBCDIC-IT", NULL,
    "EBCDIC-IT-translit", NULL,
    "EBCDIC-JP-E", NULL,
    "EBCDIC-JP-E-translit", NULL,
    "EBCDIC-JP-KANA", NULL,
    "EBCDIC-JP-KANA-translit", NULL,
    "EBCDIC-PT", NULL,
    "EBCDIC-PT-translit", NULL,
    "EBCDIC-UK", NULL,
    "EBCDIC-UK-translit", NULL,
    "EBCDIC-US", NULL,
    "EBCDIC-US-translit", NULL,
    "EBCDICATDE", NULL,
    "EBCDICATDE-translit", NULL,
    "EBCDICATDEA", NULL,
    "EBCDICATDEA-translit", NULL,
    "EBCDICCAFR", NULL,
    "EBCDICCAFR-translit", NULL,
    "EBCDICDKNO", NULL,
    "EBCDICDKNO-translit", NULL,
    "EBCDICDKNOA", NULL,
    "EBCDICDKNOA-translit", NULL,
    "EBCDICES", NULL,
    "EBCDICES-translit", NULL,
    "EBCDICESA", NULL,
    "EBCDICESA-translit", NULL,
    "EBCDICESS", NULL,
    "EBCDICESS-translit", NULL,
    "EBCDICFISE", NULL,
    "EBCDICFISE-translit", NULL,
    "EBCDICFISEA", NULL,
    "EBCDICFISEA-translit", NULL,
    "EBCDICFR", NULL,
    "EBCDICFR-translit", NULL,
    "EBCDICISFRISS", NULL,
    "EBCDICISFRISS-translit", NULL,
    "EBCDICIT", NULL,
    "EBCDICIT-translit", NULL,
    "EBCDICPT", NULL,
    "EBCDICPT-translit", NULL,
    "EBCDICUK", NULL,
    "EBCDICUK-translit", NULL,
    "EBCDICUS", NULL,
    "EBCDICUS-translit", NULL,
    "ECMA-114", NULL,
    "ECMA-114-translit", NULL,
    "ECMA-118", NULL,
    "ECMA-118-translit", NULL,
    "ECMA-128", NULL,
    "ECMA-128-translit", NULL,
    "ECMA-CYRILLIC", NULL,
    "ECMA-CYRILLIC-translit", NULL,
    "ECMACYRILLIC", NULL,
    "ECMACYRILLIC-translit", NULL,
    "ELOT_928", NULL,
    "ELOT_928-translit", NULL,
    "ES", NULL,
    "ES-translit", NULL,
    "ES2", NULL,
    "ES2-translit", NULL,
    "EUC-CN", NULL,
    "EUC-CN-translit", NULL,
    "EUC-JISX0213", NULL,
    "EUC-JISX0213-translit", NULL,
    "EUC-JP-MS", NULL,
    "EUC-JP-MS-translit", NULL,
    "EUC-JP", NULL,
    "EUC-JP-translit", NULL,
    "EUC-KR", NULL,
    "EUC-KR-translit", NULL,
    "EUC-TW", NULL,
    "EUC-TW-translit", NULL,
    "EUCCN", NULL,
    "EUCCN-translit", NULL,
    "EUCJP-MS", NULL,
    "EUCJP-MS-translit", NULL,
    "EUCJP-OPEN", NULL,
    "EUCJP-OPEN-translit", NULL,
    "EUCJP-WIN", NULL,
    "EUCJP-WIN-translit", NULL,
    "EUCJP", NULL,
    "EUCJP-translit", NULL,
    "EUCKR", NULL,
    "EUCKR-translit", NULL,
    "EUCTW", NULL,
    "EUCTW-translit", NULL,
    "FI", NULL,
    "FI-translit", NULL,
    "FR", NULL,
    "FR-translit", NULL,
    "GB", NULL,
    "GB-translit", NULL,
    "GB2312", NULL,
    "GB2312-translit", NULL,
    "GB13000", NULL,
    "GB13000-translit", NULL,
    "GB18030", NULL,
    "GB18030-translit", NULL,
    "GBK", NULL,
    "GBK-translit", NULL,
    "GB_1988-80", NULL,
    "GB_1988-80-translit", NULL,
    "GB_198880", NULL,
    "GB_198880-translit", NULL,
    "GEORGIAN-ACADEMY", NULL,
    "GEORGIAN-ACADEMY-translit", NULL,
    "GEORGIAN-PS", NULL,
    "GEORGIAN-PS-translit", NULL,
    "GOST_19768-74", NULL,
    "GOST_19768-74-translit", NULL,
    "GOST_19768", NULL,
    "GOST_19768-translit", NULL,
    "GOST_1976874", NULL,
    "GOST_1976874-translit", NULL,
    "GREEK-CCITT", NULL,
    "GREEK-CCITT-translit", NULL,
    "GREEK", NULL,
    "GREEK-translit", NULL,
    "GREEK7-OLD", NULL,
    "GREEK7-OLD-translit", NULL,
    "GREEK7", NULL,
    "GREEK7-translit", NULL,
    "GREEK7OLD", NULL,
    "GREEK7OLD-translit", NULL,
    "GREEK8", NULL,
    "GREEK8-translit", NULL,
    "GREEKCCITT", NULL,
    "GREEKCCITT-translit", NULL,
    "HEBREW", NULL,
    "HEBREW-translit", NULL,
    "HP-GREEK8", NULL,
    "HP-GREEK8-translit", NULL,
    "HP-ROMAN8", NULL,
    "HP-ROMAN8-translit", NULL,
    "HP-ROMAN9", NULL,
    "HP-ROMAN9-translit", NULL,
    "HP-THAI8", NULL,
    "HP-THAI8-translit", NULL,
    "HP-TURKISH8", NULL,
    "HP-TURKISH8-translit", NULL,
    "HPGREEK8", NULL,
    "HPGREEK8-translit", NULL,
    "HPROMAN8", NULL,
    "HPROMAN8-translit", NULL,
    "HPROMAN9", NULL,
    "HPROMAN9-translit", NULL,
    "HPTHAI8", NULL,
    "HPTHAI8-translit", NULL,
    "HPTURKISH8", NULL,
    "HPTURKISH8-translit", NULL,
    "HU", NULL,
    "HU-translit", NULL,
    "IBM-803", NULL,
    "IBM-803-translit", NULL,
    "IBM-856", NULL,
    "IBM-856-translit", NULL,
    "IBM-901", NULL,
    "IBM-901-translit", NULL,
    "IBM-902", NULL,
    "IBM-902-translit", NULL,
    "IBM-921", NULL,
    "IBM-921-translit", NULL,
    "IBM-922", NULL,
    "IBM-922-translit", NULL,
    "IBM-930", NULL,
    "IBM-930-translit", NULL,
    "IBM-932", NULL,
    "IBM-932-translit", NULL,
    "IBM-933", NULL,
    "IBM-933-translit", NULL,
    "IBM-935", NULL,
    "IBM-935-translit", NULL,
    "IBM-937", NULL,
    "IBM-937-translit", NULL,
    "IBM-939", NULL,
    "IBM-939-translit", NULL,
    "IBM-943", NULL,
    "IBM-943-translit", NULL,
    "IBM-1008", NULL,
    "IBM-1008-translit", NULL,
    "IBM-1025", NULL,
    "IBM-1025-translit", NULL,
    "IBM-1046", NULL,
    "IBM-1046-translit", NULL,
    "IBM-1047", NULL,
    "IBM-1047-translit", NULL,
    "IBM-1097", NULL,
    "IBM-1097-translit", NULL,
    "IBM-1112", NULL,
    "IBM-1112-translit", NULL,
    "IBM-1122", NULL,
    "IBM-1122-translit", NULL,
    "IBM-1123", NULL,
    "IBM-1123-translit", NULL,
    "IBM-1124", NULL,
    "IBM-1124-translit", NULL,
    "IBM-1129", NULL,
    "IBM-1129-translit", NULL,
    "IBM-1130", NULL,
    "IBM-1130-translit", NULL,
    "IBM-1132", NULL,
    "IBM-1132-translit", NULL,
    "IBM-1133", NULL,
    "IBM-1133-translit", NULL,
    "IBM-1137", NULL,
    "IBM-1137-translit", NULL,
    "IBM-1140", NULL,
    "IBM-1140-translit", NULL,
    "IBM-1141", NULL,
    "IBM-1141-translit", NULL,
    "IBM-1142", NULL,
    "IBM-1142-translit", NULL,
    "IBM-1143", NULL,
    "IBM-1143-translit", NULL,
    "IBM-1144", NULL,
    "IBM-1144-translit", NULL,
    "IBM-1145", NULL,
    "IBM-1145-translit", NULL,
    "IBM-1146", NULL,
    "IBM-1146-translit", NULL,
    "IBM-1147", NULL,
    "IBM-1147-translit", NULL,
    "IBM-1148", NULL,
    "IBM-1148-translit", NULL,
    "IBM-1149", NULL,
    "IBM-1149-translit", NULL,
    "IBM-1153", NULL,
    "IBM-1153-translit", NULL,
    "IBM-1154", NULL,
    "IBM-1154-translit", NULL,
    "IBM-1155", NULL,
    "IBM-1155-translit", NULL,
    "IBM-1156", NULL,
    "IBM-1156-translit", NULL,
    "IBM-1157", NULL,
    "IBM-1157-translit", NULL,
    "IBM-1158", NULL,
    "IBM-1158-translit", NULL,
    "IBM-1160", NULL,
    "IBM-1160-translit", NULL,
    "IBM-1161", NULL,
    "IBM-1161-translit", NULL,
    "IBM-1162", NULL,
    "IBM-1162-translit", NULL,
    "IBM-1163", NULL,
    "IBM-1163-translit", NULL,
    "IBM-1164", NULL,
    "IBM-1164-translit", NULL,
    "IBM-1166", NULL,
    "IBM-1166-translit", NULL,
    "IBM-1167", NULL,
    "IBM-1167-translit", NULL,
    "IBM-1364", NULL,
    "IBM-1364-translit", NULL,
    "IBM-1371", NULL,
    "IBM-1371-translit", NULL,
    "IBM-1388", NULL,
    "IBM-1388-translit", NULL,
    "IBM-1390", NULL,
    "IBM-1390-translit", NULL,
    "IBM-1399", NULL,
    "IBM-1399-translit", NULL,
    "IBM-4517", NULL,
    "IBM-4517-translit", NULL,
    "IBM-4899", NULL,
    "IBM-4899-translit", NULL,
    "IBM-4909", NULL,
    "IBM-4909-translit", NULL,
    "IBM-4971", NULL,
    "IBM-4971-translit", NULL,
    "IBM-5347", NULL,
    "IBM-5347-translit", NULL,
    "IBM-9030", NULL,
    "IBM-9030-translit", NULL,
    "IBM-9066", NULL,
    "IBM-9066-translit", NULL,
    "IBM-9448", NULL,
    "IBM-9448-translit", NULL,
    "IBM-12712", NULL,
    "IBM-12712-translit", NULL,
    "IBM-16804", NULL,
    "IBM-16804-translit", NULL,
    "IBM037", NULL,
    "IBM037-translit", NULL,
    "IBM038", NULL,
    "IBM038-translit", NULL,
    "IBM256", NULL,
    "IBM256-translit", NULL,
    "IBM273", NULL,
    "IBM273-translit", NULL,
    "IBM274", NULL,
    "IBM274-translit", NULL,
    "IBM275", NULL,
    "IBM275-translit", NULL,
    "IBM277", NULL,
    "IBM277-translit", NULL,
    "IBM278", NULL,
    "IBM278-translit", NULL,
    "IBM280", NULL,
    "IBM280-translit", NULL,
    "IBM281", NULL,
    "IBM281-translit", NULL,
    "IBM284", NULL,
    "IBM284-translit", NULL,
    "IBM285", NULL,
    "IBM285-translit", NULL,
    "IBM290", NULL,
    "IBM290-translit", NULL,
    "IBM297", NULL,
    "IBM297-translit", NULL,
    "IBM367", NULL,
    "IBM367-translit", NULL,
    "IBM420", NULL,
    "IBM420-translit", NULL,
    "IBM423", NULL,
    "IBM423-translit", NULL,
    "IBM424", NULL,
    "IBM424-translit", NULL,
    "IBM437", NULL,
    "IBM437-translit", NULL,
    "IBM500", NULL,
    "IBM500-translit", NULL,
    "IBM775", NULL,
    "IBM775-translit", NULL,
    "IBM803", NULL,
    "IBM803-translit", NULL,
    "IBM813", NULL,
    "IBM813-translit", NULL,
    "IBM819", NULL,
    "IBM819-translit", NULL,
    "IBM848", NULL,
    "IBM848-translit", NULL,
    "IBM850", NULL,
    "IBM850-translit", NULL,
    "IBM851", NULL,
    "IBM851-translit", NULL,
    "IBM852", NULL,
    "IBM852-translit", NULL,
    "IBM855", NULL,
    "IBM855-translit", NULL,
    "IBM856", NULL,
    "IBM856-translit", NULL,
    "IBM857", NULL,
    "IBM857-translit", NULL,
    "IBM858", NULL,
    "IBM858-translit", NULL,
    "IBM860", NULL,
    "IBM860-translit", NULL,
    "IBM861", NULL,
    "IBM861-translit", NULL,
    "IBM862", NULL,
    "IBM862-translit", NULL,
    "IBM863", NULL,
    "IBM863-translit", NULL,
    "IBM864", NULL,
    "IBM864-translit", NULL,
    "IBM865", NULL,
    "IBM865-translit", NULL,
    "IBM866", NULL,
    "IBM866-translit", NULL,
    "IBM866NAV", NULL,
    "IBM866NAV-translit", NULL,
    "IBM868", NULL,
    "IBM868-translit", NULL,
    "IBM869", NULL,
    "IBM869-translit", NULL,
    "IBM870", NULL,
    "IBM870-translit", NULL,
    "IBM871", NULL,
    "IBM871-translit", NULL,
    "IBM874", NULL,
    "IBM874-translit", NULL,
    "IBM875", NULL,
    "IBM875-translit", NULL,
    "IBM880", NULL,
    "IBM880-translit", NULL,
    "IBM891", NULL,
    "IBM891-translit", NULL,
    "IBM901", NULL,
    "IBM901-translit", NULL,
    "IBM902", NULL,
    "IBM902-translit", NULL,
    "IBM903", NULL,
    "IBM903-translit", NULL,
    "IBM904", NULL,
    "IBM904-translit", NULL,
    "IBM905", NULL,
    "IBM905-translit", NULL,
    "IBM912", NULL,
    "IBM912-translit", NULL,
    "IBM915", NULL,
    "IBM915-translit", NULL,
    "IBM916", NULL,
    "IBM916-translit", NULL,
    "IBM918", NULL,
    "IBM918-translit", NULL,
    "IBM920", NULL,
    "IBM920-translit", NULL,
    "IBM921", NULL,
    "IBM921-translit", NULL,
    "IBM922", NULL,
    "IBM922-translit", NULL,
    "IBM930", NULL,
    "IBM930-translit", NULL,
    "IBM932", NULL,
    "IBM932-translit", NULL,
    "IBM933", NULL,
    "IBM933-translit", NULL,
    "IBM935", NULL,
    "IBM935-translit", NULL,
    "IBM937", NULL,
    "IBM937-translit", NULL,
    "IBM939", NULL,
    "IBM939-translit", NULL,
    "IBM943", NULL,
    "IBM943-translit", NULL,
    "IBM1004", NULL,
    "IBM1004-translit", NULL,
    "IBM1008", NULL,
    "IBM1008-translit", NULL,
    "IBM1025", NULL,
    "IBM1025-translit", NULL,
    "IBM1026", NULL,
    "IBM1026-translit", NULL,
    "IBM1046", NULL,
    "IBM1046-translit", NULL,
    "IBM1047", NULL,
    "IBM1047-translit", NULL,
    "IBM1089", NULL,
    "IBM1089-translit", NULL,
    "IBM1097", NULL,
    "IBM1097-translit", NULL,
    "IBM1112", NULL,
    "IBM1112-translit", NULL,
    "IBM1122", NULL,
    "IBM1122-translit", NULL,
    "IBM1123", NULL,
    "IBM1123-translit", NULL,
    "IBM1124", NULL,
    "IBM1124-translit", NULL,
    "IBM1129", NULL,
    "IBM1129-translit", NULL,
    "IBM1130", NULL,
    "IBM1130-translit", NULL,
    "IBM1132", NULL,
    "IBM1132-translit", NULL,
    "IBM1133", NULL,
    "IBM1133-translit", NULL,
    "IBM1137", NULL,
    "IBM1137-translit", NULL,
    "IBM1140", NULL,
    "IBM1140-translit", NULL,
    "IBM1141", NULL,
    "IBM1141-translit", NULL,
    "IBM1142", NULL,
    "IBM1142-translit", NULL,
    "IBM1143", NULL,
    "IBM1143-translit", NULL,
    "IBM1144", NULL,
    "IBM1144-translit", NULL,
    "IBM1145", NULL,
    "IBM1145-translit", NULL,
    "IBM1146", NULL,
    "IBM1146-translit", NULL,
    "IBM1147", NULL,
    "IBM1147-translit", NULL,
    "IBM1148", NULL,
    "IBM1148-translit", NULL,
    "IBM1149", NULL,
    "IBM1149-translit", NULL,
    "IBM1153", NULL,
    "IBM1153-translit", NULL,
    "IBM1154", NULL,
    "IBM1154-translit", NULL,
    "IBM1155", NULL,
    "IBM1155-translit", NULL,
    "IBM1156", NULL,
    "IBM1156-translit", NULL,
    "IBM1157", NULL,
    "IBM1157-translit", NULL,
    "IBM1158", NULL,
    "IBM1158-translit", NULL,
    "IBM1160", NULL,
    "IBM1160-translit", NULL,
    "IBM1161", NULL,
    "IBM1161-translit", NULL,
    "IBM1162", NULL,
    "IBM1162-translit", NULL,
    "IBM1163", NULL,
    "IBM1163-translit", NULL,
    "IBM1164", NULL,
    "IBM1164-translit", NULL,
    "IBM1166", NULL,
    "IBM1166-translit", NULL,
    "IBM1167", NULL,
    "IBM1167-translit", NULL,
    "IBM1364", NULL,
    "IBM1364-translit", NULL,
    "IBM1371", NULL,
    "IBM1371-translit", NULL,
    "IBM1388", NULL,
    "IBM1388-translit", NULL,
    "IBM1390", NULL,
    "IBM1390-translit", NULL,
    "IBM1399", NULL,
    "IBM1399-translit", NULL,
    "IBM4517", NULL,
    "IBM4517-translit", NULL,
    "IBM4899", NULL,
    "IBM4899-translit", NULL,
    "IBM4909", NULL,
    "IBM4909-translit", NULL,
    "IBM4971", NULL,
    "IBM4971-translit", NULL,
    "IBM5347", NULL,
    "IBM5347-translit", NULL,
    "IBM9030", NULL,
    "IBM9030-translit", NULL,
    "IBM9066", NULL,
    "IBM9066-translit", NULL,
    "IBM9448", NULL,
    "IBM9448-translit", NULL,
    "IBM12712", NULL,
    "IBM12712-translit", NULL,
    "IBM16804", NULL,
    "IBM16804-translit", NULL,
    "IEC_P27-1", NULL,
    "IEC_P27-1-translit", NULL,
    "IEC_P271", NULL,
    "IEC_P271-translit", NULL,
    "INIS-8", NULL,
    "INIS-8-translit", NULL,
    "INIS-CYRILLIC", NULL,
    "INIS-CYRILLIC-translit", NULL,
    "INIS", NULL,
    "INIS-translit", NULL,
    "INIS8", NULL,
    "INIS8-translit", NULL,
    "INISCYRILLIC", NULL,
    "INISCYRILLIC-translit", NULL,
    "ISIRI-3342", NULL,
    "ISIRI-3342-translit", NULL,
    "ISIRI3342", NULL,
    "ISIRI3342-translit", NULL,
    "ISO-2022-CN-EXT", NULL,
    "ISO-2022-CN-EXT-translit", NULL,
    "ISO-2022-CN", NULL,
    "ISO-2022-CN-translit", NULL,
    "ISO-2022-JP-2", NULL,
    "ISO-2022-JP-2-translit", NULL,
    "ISO-2022-JP-3", NULL,
    "ISO-2022-JP-3-translit", NULL,
    "ISO-2022-JP", NULL,
    "ISO-2022-JP-translit", NULL,
    "ISO-2022-KR", NULL,
    "ISO-2022-KR-translit", NULL,
    "ISO-8859-1", NULL,
    "ISO-8859-1-translit", NULL,
    "ISO-8859-2", NULL,
    "ISO-8859-2-translit", NULL,
    "ISO-8859-3", NULL,
    "ISO-8859-3-translit", NULL,
    "ISO-8859-4", NULL,
    "ISO-8859-4-translit", NULL,
    "ISO-8859-5", NULL,
    "ISO-8859-5-translit", NULL,
    "ISO-8859-6", NULL,
    "ISO-8859-6-translit", NULL,
    "ISO-8859-7", NULL,
    "ISO-8859-7-translit", NULL,
    "ISO-8859-8", NULL,
    "ISO-8859-8-translit", NULL,
    "ISO-8859-9", NULL,
    "ISO-8859-9-translit", NULL,
    "ISO-8859-9E", NULL,
    "ISO-8859-9E-translit", NULL,
    "ISO-8859-10", NULL,
    "ISO-8859-10-translit", NULL,
    "ISO-8859-11", NULL,
    "ISO-8859-11-translit", NULL,
    "ISO-8859-13", NULL,
    "ISO-8859-13-translit", NULL,
    "ISO-8859-14", NULL,
    "ISO-8859-14-translit", NULL,
    "ISO-8859-15", NULL,
    "ISO-8859-15-translit", NULL,
    "ISO-8859-16", NULL,
    "ISO-8859-16-translit", NULL,
    "ISO-10646", NULL,
    "ISO-10646-translit", NULL,
    "UCS2", NULL,
    "UCS2-translit", NULL,
    "UTF-8", NULL,
    "UTF-8-translit", NULL,
    "UTF8", NULL,
    "UTF8-translit", NULL,
    "ISO-CELTIC", NULL,
    "ISO-CELTIC-translit", NULL,
    "ISO-IR-4", NULL,
    "ISO-IR-4-translit", NULL,
    "ISO-IR-6", NULL,
    "ISO-IR-6-translit", NULL,
    "ISO-IR-8-1", NULL,
    "ISO-IR-8-1-translit", NULL,
    "ISO-IR-9-1", NULL,
    "ISO-IR-9-1-translit", NULL,
    "ISO-IR-10", NULL,
    "ISO-IR-10-translit", NULL,
    "ISO-IR-11", NULL,
    "ISO-IR-11-translit", NULL,
    "ISO-IR-14", NULL,
    "ISO-IR-14-translit", NULL,
    "ISO-IR-15", NULL,
    "ISO-IR-15-translit", NULL,
    "ISO-IR-16", NULL,
    "ISO-IR-16-translit", NULL,
    "ISO-IR-17", NULL,
    "ISO-IR-17-translit", NULL,
    "ISO-IR-18", NULL,
    "ISO-IR-18-translit", NULL,
    "ISO-IR-19", NULL,
    "ISO-IR-19-translit", NULL,
    "ISO-IR-21", NULL,
    "ISO-IR-21-translit", NULL,
    "ISO-IR-25", NULL,
    "ISO-IR-25-translit", NULL,
    "ISO-IR-27", NULL,
    "ISO-IR-27-translit", NULL,
    "ISO-IR-37", NULL,
    "ISO-IR-37-translit", NULL,
    "ISO-IR-49", NULL,
    "ISO-IR-49-translit", NULL,
    "ISO-IR-50", NULL,
    "ISO-IR-50-translit", NULL,
    "ISO-IR-51", NULL,
    "ISO-IR-51-translit", NULL,
    "ISO-IR-54", NULL,
    "ISO-IR-54-translit", NULL,
    "ISO-IR-55", NULL,
    "ISO-IR-55-translit", NULL,
    "ISO-IR-57", NULL,
    "ISO-IR-57-translit", NULL,
    "ISO-IR-60", NULL,
    "ISO-IR-60-translit", NULL,
    "ISO-IR-61", NULL,
    "ISO-IR-61-translit", NULL,
    "ISO-IR-69", NULL,
    "ISO-IR-69-translit", NULL,
    "ISO-IR-84", NULL,
    "ISO-IR-84-translit", NULL,
    "ISO-IR-85", NULL,
    "ISO-IR-85-translit", NULL,
    "ISO-IR-86", NULL,
    "ISO-IR-86-translit", NULL,
    "ISO-IR-88", NULL,
    "ISO-IR-88-translit", NULL,
    "ISO-IR-89", NULL,
    "ISO-IR-89-translit", NULL,
    "ISO-IR-90", NULL,
    "ISO-IR-90-translit", NULL,
    "ISO-IR-92", NULL,
    "ISO-IR-92-translit", NULL,
    "ISO-IR-98", NULL,
    "ISO-IR-98-translit", NULL,
    "ISO-IR-99", NULL,
    "ISO-IR-99-translit", NULL,
    "ISO-IR-100", NULL,
    "ISO-IR-100-translit", NULL,
    "ISO-IR-101", NULL,
    "ISO-IR-101-translit", NULL,
    "ISO-IR-103", NULL,
    "ISO-IR-103-translit", NULL,
    "ISO-IR-109", NULL,
    "ISO-IR-109-translit", NULL,
    "ISO-IR-110", NULL,
    "ISO-IR-110-translit", NULL,
    "ISO-IR-111", NULL,
    "ISO-IR-111-translit", NULL,
    "ISO-IR-121", NULL,
    "ISO-IR-121-translit", NULL,
    "ISO-IR-122", NULL,
    "ISO-IR-122-translit", NULL,
    "ISO-IR-126", NULL,
    "ISO-IR-126-translit", NULL,
    "ISO-IR-127", NULL,
    "ISO-IR-127-translit", NULL,
    "ISO-IR-138", NULL,
    "ISO-IR-138-translit", NULL,
    "ISO-IR-139", NULL,
    "ISO-IR-139-translit", NULL,
    "ISO-IR-141", NULL,
    "ISO-IR-141-translit", NULL,
    "ISO-IR-143", NULL,
    "ISO-IR-143-translit", NULL,
    "ISO-IR-144", NULL,
    "ISO-IR-144-translit", NULL,
    "ISO-IR-148", NULL,
    "ISO-IR-148-translit", NULL,
    "ISO-IR-150", NULL,
    "ISO-IR-150-translit", NULL,
    "ISO-IR-151", NULL,
    "ISO-IR-151-translit", NULL,
    "ISO-IR-153", NULL,
    "ISO-IR-153-translit", NULL,
    "ISO-IR-155", NULL,
    "ISO-IR-155-translit", NULL,
    "ISO-IR-156", NULL,
    "ISO-IR-156-translit", NULL,
    "ISO-IR-157", NULL,
    "ISO-IR-157-translit", NULL,
    "ISO-IR-166", NULL,
    "ISO-IR-166-translit", NULL,
    "ISO-IR-179", NULL,
    "ISO-IR-179-translit", NULL,
    "ISO-IR-193", NULL,
    "ISO-IR-193-translit", NULL,
    "ISO-IR-197", NULL,
    "ISO-IR-197-translit", NULL,
    "ISO-IR-199", NULL,
    "ISO-IR-199-translit", NULL,
    "ISO-IR-203", NULL,
    "ISO-IR-203-translit", NULL,
    "ISO-IR-209", NULL,
    "ISO-IR-209-translit", NULL,
    "ISO-IR-226", NULL,
    "ISO-IR-226-translit", NULL,
    "TR_11548-1", NULL,
    "TR_11548-1-translit", NULL,
    "ISO646-CA", NULL,
    "ISO646-CA-translit", NULL,
    "ISO646-CA2", NULL,
    "ISO646-CA2-translit", NULL,
    "ISO646-CN", NULL,
    "ISO646-CN-translit", NULL,
    "ISO646-CU", NULL,
    "ISO646-CU-translit", NULL,
    "ISO646-DE", NULL,
    "ISO646-DE-translit", NULL,
    "ISO646-DK", NULL,
    "ISO646-DK-translit", NULL,
    "ISO646-ES", NULL,
    "ISO646-ES-translit", NULL,
    "ISO646-ES2", NULL,
    "ISO646-ES2-translit", NULL,
    "ISO646-FI", NULL,
    "ISO646-FI-translit", NULL,
    "ISO646-FR", NULL,
    "ISO646-FR-translit", NULL,
    "ISO646-FR1", NULL,
    "ISO646-FR1-translit", NULL,
    "ISO646-GB", NULL,
    "ISO646-GB-translit", NULL,
    "ISO646-HU", NULL,
    "ISO646-HU-translit", NULL,
    "ISO646-IT", NULL,
    "ISO646-IT-translit", NULL,
    "ISO646-JP-OCR-B", NULL,
    "ISO646-JP-OCR-B-translit", NULL,
    "ISO646-JP", NULL,
    "ISO646-JP-translit", NULL,
    "ISO646-KR", NULL,
    "ISO646-KR-translit", NULL,
    "ISO646-NO", NULL,
    "ISO646-NO-translit", NULL,
    "ISO646-NO2", NULL,
    "ISO646-NO2-translit", NULL,
    "ISO646-PT", NULL,
    "ISO646-PT-translit", NULL,
    "ISO646-PT2", NULL,
    "ISO646-PT2-translit", NULL,
    "ISO646-SE", NULL,
    "ISO646-SE-translit", NULL,
    "ISO646-SE2", NULL,
    "ISO646-SE2-translit", NULL,
    "ISO646-US", NULL,
    "ISO646-US-translit", NULL,
    "ISO646-YU", NULL,
    "ISO646-YU-translit", NULL,
    "ISO2022CN", NULL,
    "ISO2022CN-translit", NULL,
    "ISO2022CNEXT", NULL,
    "ISO2022CNEXT-translit", NULL,
    "ISO2022JP", NULL,
    "ISO2022JP-translit", NULL,
    "ISO2022JP2", NULL,
    "ISO2022JP2-translit", NULL,
    "ISO2022KR", NULL,
    "ISO2022KR-translit", NULL,
    "ISO6937", NULL,
    "ISO6937-translit", NULL,
    "ISO8859-1", NULL,
    "ISO8859-1-translit", NULL,
    "ISO8859-2", NULL,
    "ISO8859-2-translit", NULL,
    "ISO8859-3", NULL,
    "ISO8859-3-translit", NULL,
    "ISO8859-4", NULL,
    "ISO8859-4-translit", NULL,
    "ISO8859-5", NULL,
    "ISO8859-5-translit", NULL,
    "ISO8859-6", NULL,
    "ISO8859-6-translit", NULL,
    "ISO8859-7", NULL,
    "ISO8859-7-translit", NULL,
    "ISO8859-8", NULL,
    "ISO8859-8-translit", NULL,
    "ISO8859-9", NULL,
    "ISO8859-9-translit", NULL,
    "ISO8859-9E", NULL,
    "ISO8859-9E-translit", NULL,
    "ISO8859-10", NULL,
    "ISO8859-10-translit", NULL,
    "ISO8859-11", NULL,
    "ISO8859-11-translit", NULL,
    "ISO8859-13", NULL,
    "ISO8859-13-translit", NULL,
    "ISO8859-14", NULL,
    "ISO8859-14-translit", NULL,
    "ISO8859-15", NULL,
    "ISO8859-15-translit", NULL,
    "ISO8859-16", NULL,
    "ISO8859-16-translit", NULL,
    "ISO11548-1", NULL,
    "ISO11548-1-translit", NULL,
    "ISO88591", NULL,
    "ISO88591-translit", NULL,
    "ISO88592", NULL,
    "ISO88592-translit", NULL,
    "ISO88593", NULL,
    "ISO88593-translit", NULL,
    "ISO88594", NULL,
    "ISO88594-translit", NULL,
    "ISO88595", NULL,
    "ISO88595-translit", NULL,
    "ISO88596", NULL,
    "ISO88596-translit", NULL,
    "ISO88597", NULL,
    "ISO88597-translit", NULL,
    "ISO88598", NULL,
    "ISO88598-translit", NULL,
    "ISO88599", NULL,
    "ISO88599-translit", NULL,
    "ISO88599E", NULL,
    "ISO88599E-translit", NULL,
    "ISO885910", NULL,
    "ISO885910-translit", NULL,
    "ISO885911", NULL,
    "ISO885911-translit", NULL,
    "ISO885913", NULL,
    "ISO885913-translit", NULL,
    "ISO885914", NULL,
    "ISO885914-translit", NULL,
    "ISO885915", NULL,
    "ISO885915-translit", NULL,
    "ISO885916", NULL,
    "ISO885916-translit", NULL,
    "ISO_646.IRV:1991", NULL,
    "ISO_646.IRV:1991-translit", NULL,
    "ISO_2033-1983", NULL,
    "ISO_2033-1983-translit", NULL,
    "ISO_2033", NULL,
    "ISO_2033-translit", NULL,
    "ISO_5427-EXT", NULL,
    "ISO_5427-EXT-translit", NULL,
    "ISO_5427", NULL,
    "ISO_5427-translit", NULL,
    "ISO_5427:1981", NULL,
    "ISO_5427:1981-translit", NULL,
    "ISO_5427EXT", NULL,
    "ISO_5427EXT-translit", NULL,
    "ISO_5428", NULL,
    "ISO_5428-translit", NULL,
    "ISO_5428:1980", NULL,
    "ISO_5428:1980-translit", NULL,
    "ISO_6937-2", NULL,
    "ISO_6937-2-translit", NULL,
    "ISO_6937-2:1983", NULL,
    "ISO_6937-2:1983-translit", NULL,
    "ISO_6937", NULL,
    "ISO_6937-translit", NULL,
    "ISO_6937:1992", NULL,
    "ISO_6937:1992-translit", NULL,
    "ISO_8859-1", NULL,
    "ISO_8859-1-translit", NULL,
    "ISO_8859-1:1987", NULL,
    "ISO_8859-1:1987-translit", NULL,
    "ISO_8859-2", NULL,
    "ISO_8859-2-translit", NULL,
    "ISO_8859-2:1987", NULL,
    "ISO_8859-2:1987-translit", NULL,
    "ISO_8859-3", NULL,
    "ISO_8859-3-translit", NULL,
    "ISO_8859-3:1988", NULL,
    "ISO_8859-3:1988-translit", NULL,
    "ISO_8859-4", NULL,
    "ISO_8859-4-translit", NULL,
    "ISO_8859-4:1988", NULL,
    "ISO_8859-4:1988-translit", NULL,
    "ISO_8859-5", NULL,
    "ISO_8859-5-translit", NULL,
    "ISO_8859-5:1988", NULL,
    "ISO_8859-5:1988-translit", NULL,
    "ISO_8859-6", NULL,
    "ISO_8859-6-translit", NULL,
    "ISO_8859-6:1987", NULL,
    "ISO_8859-6:1987-translit", NULL,
    "ISO_8859-7", NULL,
    "ISO_8859-7-translit", NULL,
    "ISO_8859-7:1987", NULL,
    "ISO_8859-7:1987-translit", NULL,
    "ISO_8859-7:2003", NULL,
    "ISO_8859-7:2003-translit", NULL,
    "ISO_8859-8", NULL,
    "ISO_8859-8-translit", NULL,
    "ISO_8859-8:1988", NULL,
    "ISO_8859-8:1988-translit", NULL,
    "ISO_8859-9", NULL,
    "ISO_8859-9-translit", NULL,
    "ISO_8859-9:1989", NULL,
    "ISO_8859-9:1989-translit", NULL,
    "ISO_8859-9E", NULL,
    "ISO_8859-9E-translit", NULL,
    "ISO_8859-10", NULL,
    "ISO_8859-10-translit", NULL,
    "ISO_8859-10:1992", NULL,
    "ISO_8859-10:1992-translit", NULL,
    "ISO_8859-14", NULL,
    "ISO_8859-14-translit", NULL,
    "ISO_8859-14:1998", NULL,
    "ISO_8859-14:1998-translit", NULL,
    "ISO_8859-15", NULL,
    "ISO_8859-15-translit", NULL,
    "ISO_8859-15:1998", NULL,
    "ISO_8859-15:1998-translit", NULL,
    "ISO_8859-16", NULL,
    "ISO_8859-16-translit", NULL,
    "ISO_8859-16:2001", NULL,
    "ISO_8859-16:2001-translit", NULL,
    "ISO_9036", NULL,
    "ISO_9036-translit", NULL,
    "ISO_10367-BOX", NULL,
    "ISO_10367-BOX-translit", NULL,
    "ISO_10367BOX", NULL,
    "ISO_10367BOX-translit", NULL,
    "ISO_11548-1", NULL,
    "ISO_11548-1-translit", NULL,
    "ISO_69372", NULL,
    "ISO_69372-translit", NULL,
    "IT", NULL,
    "IT-translit", NULL,
    "JIS_C6220-1969-RO", NULL,
    "JIS_C6220-1969-RO-translit", NULL,
    "JIS_C6229-1984-B", NULL,
    "JIS_C6229-1984-B-translit", NULL,
    "JIS_C62201969RO", NULL,
    "JIS_C62201969RO-translit", NULL,
    "JIS_C62291984B", NULL,
    "JIS_C62291984B-translit", NULL,
    "JOHAB", NULL,
    "JOHAB-translit", NULL,
    "JP-OCR-B", NULL,
    "JP-OCR-B-translit", NULL,
    "JP", NULL,
    "JP-translit", NULL,
    "JS", NULL,
    "JS-translit", NULL,
    "JUS_I.B1.002", NULL,
    "JUS_I.B1.002-translit", NULL,
    "KOI-7", NULL,
    "KOI-7-translit", NULL,
    "KOI-8", NULL,
    "KOI-8-translit", NULL,
    "KOI8-R", NULL,
    "KOI8-R-translit", NULL,
    "KOI8-RU", NULL,
    "KOI8-RU-translit", NULL,
    "KOI8-T", NULL,
    "KOI8-T-translit", NULL,
    "KOI8-U", NULL,
    "KOI8-U-translit", NULL,
    "KOI8", NULL,
    "KOI8-translit", NULL,
    "KOI8R", NULL,
    "KOI8R-translit", NULL,
    "KOI8U", NULL,
    "KOI8U-translit", NULL,
    "KSC5636", NULL,
    "KSC5636-translit", NULL,
    "L1", NULL,
    "L1-translit", NULL,
    "L2", NULL,
    "L2-translit", NULL,
    "L3", NULL,
    "L3-translit", NULL,
    "L4", NULL,
    "L4-translit", NULL,
    "L5", NULL,
    "L5-translit", NULL,
    "L6", NULL,
    "L6-translit", NULL,
    "L7", NULL,
    "L7-translit", NULL,
    "L8", NULL,
    "L8-translit", NULL,
    "L10", NULL,
    "L10-translit", NULL,
    "LATIN-9", NULL,
    "LATIN-9-translit", NULL,
    "LATIN-GREEK-1", NULL,
    "LATIN-GREEK-1-translit", NULL,
    "LATIN-GREEK", NULL,
    "LATIN-GREEK-translit", NULL,
    "LATIN1", NULL,
    "LATIN1-translit", NULL,
    "LATIN2", NULL,
    "LATIN2-translit", NULL,
    "LATIN3", NULL,
    "LATIN3-translit", NULL,
    "LATIN4", NULL,
    "LATIN4-translit", NULL,
    "LATIN5", NULL,
    "LATIN5-translit", NULL,
    "LATIN6", NULL,
    "LATIN6-translit", NULL,
    "LATIN7", NULL,
    "LATIN7-translit", NULL,
    "LATIN8", NULL,
    "LATIN8-translit", NULL,
    "LATIN9", NULL,
    "LATIN9-translit", NULL,
    "LATIN10", NULL,
    "LATIN10-translit", NULL,
    "LATINGREEK", NULL,
    "LATINGREEK-translit", NULL,
    "LATINGREEK1", NULL,
    "LATINGREEK1-translit", NULL,
    "MAC-CENTRALEUROPE", NULL,
    "MAC-CENTRALEUROPE-translit", NULL,
    "MAC-CYRILLIC", NULL,
    "MAC-CYRILLIC-translit", NULL,
    "MAC-IS", NULL,
    "MAC-IS-translit", NULL,
    "MAC-SAMI", NULL,
    "MAC-SAMI-translit", NULL,
    "MAC-UK", NULL,
    "MAC-UK-translit", NULL,
    "MAC", NULL,
    "MAC-translit", NULL,
    "MACCYRILLIC", NULL,
    "MACCYRILLIC-translit", NULL,
    "MACINTOSH", NULL,
    "MACINTOSH-translit", NULL,
    "MACIS", NULL,
    "MACIS-translit", NULL,
    "MACUK", NULL,
    "MACUK-translit", NULL,
    "MACUKRAINIAN", NULL,
    "MACUKRAINIAN-translit", NULL,
    "MIK", NULL,
    "MIK-translit", NULL,
    "MS-ANSI", NULL,
    "MS-ANSI-translit", NULL,
    "MS-ARAB", NULL,
    "MS-ARAB-translit", NULL,
    "MS-CYRL", NULL,
    "MS-CYRL-translit", NULL,
    "MS-EE", NULL,
    "MS-EE-translit", NULL,
    "MS-GREEK", NULL,
    "MS-GREEK-translit", NULL,
    "MS-HEBR", NULL,
    "MS-HEBR-translit", NULL,
    "MS-MAC-CYRILLIC", NULL,
    "MS-MAC-CYRILLIC-translit", NULL,
    "MS-TURK", NULL,
    "MS-TURK-translit", NULL,
    "MS932", NULL,
    "MS932-translit", NULL,
    "MS936", NULL,
    "MS936-translit", NULL,
    "MSCP949", NULL,
    "MSCP949-translit", NULL,
    "MSCP1361", NULL,
    "MSCP1361-translit", NULL,
    "MSMACCYRILLIC", NULL,
    "MSMACCYRILLIC-translit", NULL,
    "MSZ_7795.3", NULL,
    "MSZ_7795.3-translit", NULL,
    "MS_KANJI", NULL,
    "MS_KANJI-translit", NULL,
    "NAPLPS", NULL,
    "NAPLPS-translit", NULL,
    "NATS-DANO", NULL,
    "NATS-DANO-translit", NULL,
    "NATS-SEFI", NULL,
    "NATS-SEFI-translit", NULL,
    "NATSDANO", NULL,
    "NATSDANO-translit", NULL,
    "NATSSEFI", NULL,
    "NATSSEFI-translit", NULL,
    "NC_NC0010", NULL,
    "NC_NC0010-translit", NULL,
    "NC_NC00-10", NULL,
    "NC_NC00-10-translit", NULL,
    "NC_NC00-10:81", NULL,
    "NC_NC00-10:81-translit", NULL,
    "NF_Z_62-010", NULL,
    "NF_Z_62-010-translit", NULL,
    "NF_Z_62-010_(1973)", NULL,
    "NF_Z_62-010_(1973)-translit", NULL,
    "NF_Z_62-010_1973", NULL,
    "NF_Z_62-010_1973-translit", NULL,
    "NF_Z_62010", NULL,
    "NF_Z_62010-translit", NULL,
    "NF_Z_62010_1973", NULL,
    "NF_Z_62010_1973-translit", NULL,
    "NO", NULL,
    "NO-translit", NULL,
    "NO2", NULL,
    "NO2-translit", NULL,
    "NS_4551-1", NULL,
    "NS_4551-1-translit", NULL,
    "NS_4551-2", NULL,
    "NS_4551-2-translit", NULL,
    "NS_45511", NULL,
    "NS_45511-translit", NULL,
    "NS_45512", NULL,
    "NS_45512-translit", NULL,
    "OS2LATIN1", NULL,
    "OS2LATIN1-translit", NULL,
    "OSF00010001", NULL,
    "OSF00010001-translit", NULL,
    "OSF00010002", NULL,
    "OSF00010002-translit", NULL,
    "OSF00010003", NULL,
    "OSF00010003-translit", NULL,
    "OSF00010004", NULL,
    "OSF00010004-translit", NULL,
    "OSF00010005", NULL,
    "OSF00010005-translit", NULL,
    "OSF00010006", NULL,
    "OSF00010006-translit", NULL,
    "OSF00010007", NULL,
    "OSF00010007-translit", NULL,
    "OSF00010008", NULL,
    "OSF00010008-translit", NULL,
    "OSF00010009", NULL,
    "OSF00010009-translit", NULL,
    "OSF0001000A", NULL,
    "OSF0001000A-translit", NULL,
    "OSF00010020", NULL,
    "OSF00010020-translit", NULL,
    "OSF00010100", NULL,
    "OSF00010100-translit", NULL,
    "OSF00010101", NULL,
    "OSF00010101-translit", NULL,
    "OSF00010102", NULL,
    "OSF00010102-translit", NULL,
    "OSF00010104", NULL,
    "OSF00010104-translit", NULL,
    "OSF00010105", NULL,
    "OSF00010105-translit", NULL,
    "OSF00010106", NULL,
    "OSF00010106-translit", NULL,
    "OSF00030010", NULL,
    "OSF00030010-translit", NULL,
    "OSF0004000A", NULL,
    "OSF0004000A-translit", NULL,
    "OSF0005000A", NULL,
    "OSF0005000A-translit", NULL,
    "OSF05010001", NULL,
    "OSF05010001-translit", NULL,
    "OSF100201A4", NULL,
    "OSF100201A4-translit", NULL,
    "OSF100201A8", NULL,
    "OSF100201A8-translit", NULL,
    "OSF100201B5", NULL,
    "OSF100201B5-translit", NULL,
    "OSF100201F4", NULL,
    "OSF100201F4-translit", NULL,
    "OSF100203B5", NULL,
    "OSF100203B5-translit", NULL,
    "OSF1002011C", NULL,
    "OSF1002011C-translit", NULL,
    "OSF1002011D", NULL,
    "OSF1002011D-translit", NULL,
    "OSF1002035D", NULL,
    "OSF1002035D-translit", NULL,
    "OSF1002035E", NULL,
    "OSF1002035E-translit", NULL,
    "OSF1002035F", NULL,
    "OSF1002035F-translit", NULL,
    "OSF1002036B", NULL,
    "OSF1002036B-translit", NULL,
    "OSF1002037B", NULL,
    "OSF1002037B-translit", NULL,
    "OSF10010001", NULL,
    "OSF10010001-translit", NULL,
    "OSF10010004", NULL,
    "OSF10010004-translit", NULL,
    "OSF10010006", NULL,
    "OSF10010006-translit", NULL,
    "OSF10020025", NULL,
    "OSF10020025-translit", NULL,
    "OSF10020111", NULL,
    "OSF10020111-translit", NULL,
    "OSF10020115", NULL,
    "OSF10020115-translit", NULL,
    "OSF10020116", NULL,
    "OSF10020116-translit", NULL,
    "OSF10020118", NULL,
    "OSF10020118-translit", NULL,
    "OSF10020122", NULL,
    "OSF10020122-translit", NULL,
    "OSF10020129", NULL,
    "OSF10020129-translit", NULL,
    "OSF10020352", NULL,
    "OSF10020352-translit", NULL,
    "OSF10020354", NULL,
    "OSF10020354-translit", NULL,
    "OSF10020357", NULL,
    "OSF10020357-translit", NULL,
    "OSF10020359", NULL,
    "OSF10020359-translit", NULL,
    "OSF10020360", NULL,
    "OSF10020360-translit", NULL,
    "OSF10020364", NULL,
    "OSF10020364-translit", NULL,
    "OSF10020365", NULL,
    "OSF10020365-translit", NULL,
    "OSF10020366", NULL,
    "OSF10020366-translit", NULL,
    "OSF10020367", NULL,
    "OSF10020367-translit", NULL,
    "OSF10020370", NULL,
    "OSF10020370-translit", NULL,
    "OSF10020387", NULL,
    "OSF10020387-translit", NULL,
    "OSF10020388", NULL,
    "OSF10020388-translit", NULL,
    "OSF10020396", NULL,
    "OSF10020396-translit", NULL,
    "OSF10020402", NULL,
    "OSF10020402-translit", NULL,
    "OSF10020417", NULL,
    "OSF10020417-translit", NULL,
    "PT", NULL,
    "PT-translit", NULL,
    "PT2", NULL,
    "PT2-translit", NULL,
    "PT154", NULL,
    "PT154-translit", NULL,
    "R8", NULL,
    "R8-translit", NULL,
    "R9", NULL,
    "R9-translit", NULL,
    "RK1048", NULL,
    "RK1048-translit", NULL,
    "ROMAN8", NULL,
    "ROMAN8-translit", NULL,
    "ROMAN9", NULL,
    "ROMAN9-translit", NULL,
    "RUSCII", NULL,
    "RUSCII-translit", NULL,
    "SE", NULL,
    "SE-translit", NULL,
    "SE2", NULL,
    "SE2-translit", NULL,
    "SEN_850200_B", NULL,
    "SEN_850200_B-translit", NULL,
    "SEN_850200_C", NULL,
    "SEN_850200_C-translit", NULL,
    "SHIFT-JIS", NULL,
    "SHIFT-JIS-translit", NULL,
    "SHIFTJISX0213", NULL,
    "SHIFTJISX0213-translit", NULL,
    "SHIFT_JIS", NULL,
    "SHIFT_JIS-translit", NULL,
    "SHIFT_JISX0213", NULL,
    "SHIFT_JISX0213-translit", NULL,
    "SJIS-OPEN", NULL,
    "SJIS-OPEN-translit", NULL,
    "SJIS-WIN", NULL,
    "SJIS-WIN-translit", NULL,
    "SJIS", NULL,
    "SJIS-translit", NULL,
    "SS636127", NULL,
    "SS636127-translit", NULL,
    "STRK1048-2002", NULL,
    "STRK1048-2002-translit", NULL,
    "ST_SEV_358-88", NULL,
    "ST_SEV_358-88-translit", NULL,
    "T.61-8BIT", NULL,
    "T.61-8BIT-translit", NULL,
    "T.61", NULL,
    "T.61-translit", NULL,
    "T.618BIT", NULL,
    "T.618BIT-translit", NULL,
    "TCVN-5712", NULL,
    "TCVN-5712-translit", NULL,
    "TCVN", NULL,
    "TCVN-translit", NULL,
    "TCVN5712-1", NULL,
    "TCVN5712-1-translit", NULL,
    "TCVN5712-1:1993", NULL,
    "TCVN5712-1:1993-translit", NULL,
    "THAI8", NULL,
    "THAI8-translit", NULL,
    "TIS-620", NULL,
    "TIS-620-translit", NULL,
    "TIS620-0", NULL,
    "TIS620-0-translit", NULL,
    "TIS620.2529-1", NULL,
    "TIS620.2529-1-translit", NULL,
    "TIS620.2533-0", NULL,
    "TIS620.2533-0-translit", NULL,
    "TIS620", NULL,
    "TIS620-translit", NULL,
    "TS-5881", NULL,
    "TS-5881-translit", NULL,
    "TSCII", NULL,
    "TSCII-translit", NULL,
    "TURKISH8", NULL,
    "TURKISH8-translit", NULL,
    "UCS-2", NULL,
    "UCS-2-translit", NULL,
    "UCS-2BE", NULL,
    "UCS-2BE-translit", NULL,
    "UCS-2LE", NULL,
    "UCS-2LE-translit", NULL,
    "UCS-4", NULL,
    "UCS-4-translit", NULL,
    "UCS-4BE", NULL,
    "UCS-4BE-translit", NULL,
    "UCS-4LE", NULL,
    "UCS-4LE-translit", NULL,
    "UHC", NULL,
    "UHC-translit", NULL,
    "UJIS", NULL,
    "UJIS-translit", NULL,
    "UK", NULL,
    "UK-translit", NULL,
    "UNICODE", NULL,
    "UNICODE-translit", NULL,
    "UNICODEBIG", NULL,
    "UNICODEBIG-translit", NULL,
    "UNICODELITTLE", NULL,
    "UNICODELITTLE-translit", NULL,
    "US-ASCII", NULL,
    "US-ASCII-translit", NULL,
    "US", NULL,
    "US-translit", NULL,
    "UTF-7-IMAP", NULL,
    "UTF-7-IMAP-translit", NULL,
    "UTF-7", NULL,
    "UTF-7-translit", NULL,
    "UTF-16", NULL,
    "UTF-16-translit", NULL,
    "UTF-16BE", NULL,
    "UTF-16BE-translit", NULL,
    "UTF-16LE", NULL,
    "UTF-16LE-translit", NULL,
    "UTF-32", NULL,
    "UTF-32-translit", NULL,
    "UTF-32BE", NULL,
    "UTF-32BE-translit", NULL,
    "UTF-32LE", NULL,
    "UTF-32LE-translit", NULL,
    "UTF7", NULL,
    "UTF7-translit", NULL,
    "UTF16", NULL,
    "UTF16-translit", NULL,
    "UTF16BE", NULL,
    "UTF16BE-translit", NULL,
    "UTF16LE", NULL,
    "UTF16LE-translit", NULL,
    "UTF32", NULL,
    "UTF32-translit", NULL,
    "UTF32BE", NULL,
    "UTF32BE-translit", NULL,
    "UTF32LE", NULL,
    "UTF32LE-translit", NULL,
    "VISCII", NULL,
    "VISCII-translit", NULL,
    "WCHAR_T", NULL,
    "WCHAR_T-translit", NULL,
    "WIN-SAMI-2", NULL,
    "WIN-SAMI-2-translit", NULL,
    "WINBALTRIM", NULL,
    "WINBALTRIM-translit", NULL,
    "WINDOWS-31J", NULL,
    "WINDOWS-31J-translit", NULL,
    "WINDOWS-874", NULL,
    "WINDOWS-874-translit", NULL,
    "WINDOWS-936", NULL,
    "WINDOWS-936-translit", NULL,
    "WINDOWS-1250", NULL,
    "WINDOWS-1250-translit", NULL,
    "WINDOWS-1251", NULL,
    "WINDOWS-1251-translit", NULL,
    "WINDOWS-1252", NULL,
    "WINDOWS-1252-translit", NULL,
    "WINDOWS-1253", NULL,
    "WINDOWS-1253-translit", NULL,
    "WINDOWS-1254", NULL,
    "WINDOWS-1254-translit", NULL,
    "WINDOWS-1255", NULL,
    "WINDOWS-1255-translit", NULL,
    "WINDOWS-1256", NULL,
    "WINDOWS-1256-translit", NULL,
    "WINDOWS-1257", NULL,
    "WINDOWS-1257-translit", NULL,
    "WINDOWS-1258", NULL,
    "WINDOWS-1258-translit", NULL,
    "WINSAMI2", NULL,
    "WINSAMI2-translit", NULL,
    "WS2", NULL,
    "WS2-translit", NULL,
    "YU", NULL,
    "YU-translit", NULL,
    NULL
  };
